/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Bancisor Mihai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 684
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "Communication.h"
#include "AD799x.h"
#include "Console.h"
/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
_FOSCSEL(FNOSC_PRIPLL);
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF  & POSCMD_XT);    /*!< Clock Switching Enabled and Fail Safe Clock Monitor is disabled*/
                                                    /*!< OSC2 Pin Function: OSC2 is Clock Output*/
                                                    /*!  Primary Oscillator Mode: XT Crystal*/
_FWDT(FWDTEN_OFF); 			            /*!< Watchdog Timer Enabled/disabled by user software*/

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/*! List of available commands */
const char* commandsList[] = {"help?",
                              "channel=",
                              "channel?",
                              "voltage?",
                              "register?"};
const char* commandsDescription[] = {
    "  -  Diplays all available commands.",
	"  -  Selects the channel that will be affected by the other commands. Accepted values: 0 ? 3 (one channel).",
    "  -  Displays the selected channel.",
    "  -  Initiates a conversion and displays the captured voltage.",
    "  -  Initiates a conversion and displays the raw data in decimal format."};
const float VDD = 3.3; /*!< VDD = 3.3V */

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
double        commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converterd value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    short  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    short  i = sizeof(localBuf) - 1;

    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > sizeof(localBuf) - 4)
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
	*buf = 0;
}
/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
int main(void)
{
    /*! Configure PLL prescaler, PLL postscaler, PLL divisor. */
    /*! Fosc = 80MHz, Fcy = 40MHz, 40MIPS. */
    PLLFBD = 38;            // M = PLLFBD + 2
    CLKDIVbits.PLLPOST = 0; // N2 = 2
    CLKDIVbits.PLLPRE = 0;  // N1 = 2

    /*! Variables holding information about the device */
    unsigned short channelsValues[4] = {0, 0, 0, 0};
    unsigned short registerValue = 0; /*!< Last value captured from convertor register */
    unsigned char currentChannel = 0; /*!< Last selected channel */
    /*! Temporary variables */
    char    tempString[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    double  voltage = 0;

    /*! Wait for PLL to lock. */
    while(OSCCONbits.COSC != 0b011);
    while(OSCCONbits.LOCK != 1);

    /*! Initialize the UART communication peripheral. */
    UART_Init(9600);
    /*! Initialize the AD799x device. */
    if(AD799x_Init())
    {
        CONSOLE_WriteString("AD799x OK");
        UART_Write(0x0D);
    }
    else
    {
        CONSOLE_WriteString("AD799x Error");
        UART_Write(0x0D);
    }
    /*! Enable channel 0. */
    AD799x_SetRegisterValue(AD7991_CHANNEL(0));
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    /*!< "help?" command*/
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))  /*!< "channel=" command*/
            {
                if((commandParam >= 0) && (commandParam <= 3))
                {
                    currentChannel = (char)commandParam;
                }
                else
                {
                    currentChannel = 0; /*!< Select channel 0*/
                }
                /*! Power on the selected channel and shutdown the others */
                AD799x_SetRegisterValue(AD7991_CHANNEL(currentChannel));
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, currentChannel, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 2) && (commandType != 0))  /*!< "channel?" command*/
            {
                /*! Send the requested value to user */
                CONSOLE_WriteString(commandsList[command - 1]);
                itoa(tempString, currentChannel, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 3) && (commandType != 0))  /*!< "voltage?" command*/
            {
                /*! Start a conversion */
                AD799x_GetConversionResult(&channelsValues[currentChannel],
                                           &currentChannel,
                                           AD7991_BITS_NO,
                                           AD7991_CHANNELS_NO);
                registerValue = channelsValues[currentChannel];
                /*! Get the input voltage from the straight binary raw data */
                voltage = ((double)registerValue / 4096) * VDD;
                /*! Send the requested value to user */
                CONSOLE_WriteString("voltage=");
                /*! Convert float to string and send it to user */
                FloatToString(tempString, voltage);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 4) && (commandType != 0))  /*!< "register?" command*/
            {
                AD799x_GetConversionResult(&channelsValues[currentChannel],
                                           &currentChannel,
                                           AD7991_BITS_NO,
                                           AD7991_CHANNELS_NO);
                registerValue = channelsValues[currentChannel];
                /*! Send the requested value to user */
                CONSOLE_WriteString("register=");
                itoa(tempString, registerValue, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
        }
        if(invalidCommand == commandsNumber)
        {
            /*! Send feedback to user */
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    }
}